@@ -72,12 +72,13 @@ extern PSI_file_key key_file_charset, key_file_cnf;
 #endif /* HAVE_PSI_INTERFACE */
 
 #ifdef SAFEMALLOC
-void *sf_malloc(size_t size);
-void *sf_realloc(void *ptr, size_t size);
+void *sf_malloc(size_t size, myf my_flags);
+void *sf_realloc(void *ptr, size_t size, myf my_flags);
 void sf_free(void *ptr);
+size_t sf_malloc_usable_size(void *ptr, my_bool *is_thread_specific);
 #else
-#define sf_malloc(X)    malloc(X)
-#define sf_realloc(X,Y) realloc(X,Y)
+#define sf_malloc(X,Y)    malloc(X)
+#define sf_realloc(X,Y,Z) realloc(X,Y)
 #define sf_free(X)      free(X)
 #endif
 
@@ -91,6 +92,34 @@ void sf_free(void *ptr);
 
 void my_error_unregister_all(void);
 
+#if !defined(O_PATH) && defined(O_EXEC) /* FreeBSD */
+#define O_PATH O_EXEC
+#endif
+
+#ifdef O_PATH
+#define HAVE_OPEN_PARENT_DIR_NOSYMLINKS
+const char *my_open_parent_dir_nosymlinks(const char *pathname, int *pdfd);
+#define NOSYMLINK_FUNCTION_BODY(AT,NOAT)                                \
+  int dfd, res;                                                         \
+  const char *filename= my_open_parent_dir_nosymlinks(pathname, &dfd);  \
+  if (filename == NULL) return -1;                                      \
+  res= AT;                                                              \
+  if (dfd >= 0) close(dfd);                                             \
+  return res;
+#elif defined(HAVE_REALPATH)
+#define NOSYMLINK_FUNCTION_BODY(AT,NOAT)                                \
+  char buf[PATH_MAX+1];                                                 \
+  if (realpath(pathname, buf) == NULL) return -1;                       \
+  if (strcmp(pathname, buf)) { errno= ENOTDIR; return -1; }             \
+  return NOAT;
+#else
+#define NOSYMLINK_FUNCTION_BODY(AT,NOAT)                                \
+  return NOAT;
+#endif
+
+#define CREATE_NOSYMLINK_FUNCTION(PROTO,AT,NOAT)                        \
+static int PROTO { NOSYMLINK_FUNCTION_BODY(AT,NOAT) }
+
 #ifdef _WIN32
 #include <sys/stat.h>
 /* my_winfile.c exports, should not be used outside mysys */
