@@ -1,7 +1,7 @@
 /************* Colblk C++ Functions Source Code File (.CPP) ************/
-/*  Name: COLBLK.CPP  Version 2.0                                      */
+/*  Name: COLBLK.CPP  Version 2.2                                      */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          1998-2013    */
+/*  (C) Copyright to the author Olivier BERTRAND          1998-2017    */
 /*                                                                     */
 /*  This file contains the COLBLK class functions.                     */
 /***********************************************************************/
@@ -23,8 +23,6 @@
 #include "xindex.h"
 #include "xtable.h"
 
-extern "C" int  trace;
-
 /***********************************************************************/
 /*  COLBLK protected constructor.                                      */
 /***********************************************************************/
@@ -38,8 +36,10 @@ COLBLK::COLBLK(PCOLDEF cdp, PTDB tdbp, int i)
   if ((Cdp = cdp)) {
     Name = cdp->Name;
     Format = cdp->F;
+    Opt = cdp->Opt;
     Long = cdp->Long;
     Precision = cdp->Precision;
+    Freq = cdp->Freq;
     Buf_Type = cdp->Buf_Type;
     ColUse |= cdp->Flags;       // Used by CONNECT
     Nullable = !!(cdp->Flags & U_NULLS);
@@ -47,8 +47,10 @@ COLBLK::COLBLK(PCOLDEF cdp, PTDB tdbp, int i)
   } else {
     Name = NULL;
     memset(&Format, 0, sizeof(FORMAT));
+    Opt = 0;
     Long = 0;
     Precision = 0;
+    Freq = 0;
     Buf_Type = TYPE_ERROR;
     Nullable = false;
     Unsigned = false;
@@ -109,12 +111,12 @@ bool COLBLK::Compare(PXOB xp)
 /***********************************************************************/
 /*  SetFormat: function used to set SELECT output format.              */
 /***********************************************************************/
-bool COLBLK::SetFormat(PGLOBAL g, FORMAT& fmt)
+bool COLBLK::SetFormat(PGLOBAL, FORMAT& fmt)
   {
   fmt = Format;
 
   if (trace > 1)
-    htrc("COLBLK: %p format=%c(%d,%d)\n", 
+    htrc("COLBLK: %p format=%c(%d,%d)\n",
          this, *fmt.Type, fmt.Length, fmt.Prec);
 
   return false;
@@ -173,7 +175,7 @@ bool COLBLK::InitValue(PGLOBAL g)
 /***********************************************************************/
 /*  SetBuffer: prepare a column block for write operation.             */
 /***********************************************************************/
-bool COLBLK::SetBuffer(PGLOBAL g, PVAL value, bool ok, bool check)
+bool COLBLK::SetBuffer(PGLOBAL g, PVAL, bool, bool)
   {
   sprintf(g->Message, MSG(UNDEFINED_AM), "SetBuffer");
   return true;
@@ -212,7 +214,7 @@ void COLBLK::WriteColumn(PGLOBAL g)
 /***********************************************************************/
 /*  Make file output of a column descriptor block.                     */
 /***********************************************************************/
-void COLBLK::Print(PGLOBAL g, FILE *f, uint n)
+void COLBLK::Print(PGLOBAL, FILE *f, uint n)
   {
   char m[64];
   int  i;
@@ -235,7 +237,7 @@ void COLBLK::Print(PGLOBAL g, FILE *f, uint n)
 /***********************************************************************/
 /*  Make string output of a column descriptor block.                   */
 /***********************************************************************/
-void COLBLK::Print(PGLOBAL g, char *ps, uint z)
+void COLBLK::Print(PGLOBAL, char *ps, uint)
   {
   sprintf(ps, "R%d.%s", To_Tdb->GetTdb_No(), Name);
   } // end of Print
@@ -288,17 +290,17 @@ void RIDBLK::ReadColumn(PGLOBAL g)
 /***********************************************************************/
 /*  FIDBLK constructor for the FILEID special column.                  */
 /***********************************************************************/
-FIDBLK::FIDBLK(PCOLUMN cp) : SPCBLK(cp)
+FIDBLK::FIDBLK(PCOLUMN cp, OPVAL op) : SPCBLK(cp), Op(op)
   {
 //Is_Key = 2; for when the MUL table indexed reading will be implemented.
   Precision = Long = _MAX_PATH;
   Buf_Type = TYPE_STRING;
   *Format.Type = 'C';
   Format.Length = Long;
-#if defined(WIN32)
+#if defined(__WIN__)
   Format.Prec = 1;          // Case insensitive
-#endif   // WIN32
-  Constant = (!((PTDBASE)To_Tdb)->GetDef()->GetMultiple() &&
+#endif   // __WIN__
+  Constant = (!To_Tdb->GetDef()->GetMultiple() &&
               To_Tdb->GetAmType() != TYPE_AM_PLG &&
               To_Tdb->GetAmType() != TYPE_AM_PLM);
   Fn = NULL;
@@ -310,12 +312,19 @@ FIDBLK::FIDBLK(PCOLUMN cp) : SPCBLK(cp)
 /***********************************************************************/
 void FIDBLK::ReadColumn(PGLOBAL g)
   {
-  if (Fn != ((PTDBASE)To_Tdb)->GetFile(g)) {
+  if (Fn != To_Tdb->GetFile(g)) {
     char filename[_MAX_PATH];
 
-    Fn = ((PTDBASE)To_Tdb)->GetFile(g);
-    PlugSetPath(filename, Fn, ((PTDBASE)To_Tdb)->GetPath());
-    Value->SetValue_psz(filename);
+    Fn = To_Tdb->GetFile(g);
+    PlugSetPath(filename, Fn, To_Tdb->GetPath());
+
+    if (Op != OP_XX) {
+      char buff[_MAX_PATH];
+
+      Value->SetValue_psz(ExtractFromPath(g, buff, filename, Op));
+    } else
+      Value->SetValue_psz(filename);
+
     } // endif Fn
 
   } // end of ReadColumn
@@ -338,7 +347,7 @@ TIDBLK::TIDBLK(PCOLUMN cp) : SPCBLK(cp)
 /***********************************************************************/
 /*  ReadColumn: what this routine does is to return the table ID.      */
 /***********************************************************************/
-void TIDBLK::ReadColumn(PGLOBAL g)
+void TIDBLK::ReadColumn(PGLOBAL)
   {
   if (Tname == NULL) {
     Tname = (char*)To_Tdb->GetName();
@@ -347,6 +356,36 @@ void TIDBLK::ReadColumn(PGLOBAL g)
 
   } // end of ReadColumn
 
+/***********************************************************************/
+/*  PRTBLK constructor for the PARTID special column.                  */
+/***********************************************************************/
+PRTBLK::PRTBLK(PCOLUMN cp) : SPCBLK(cp)
+  {
+//Is_Key = 2; for when the MUL table indexed reading will be implemented.
+  Precision = Long = 64;
+  Buf_Type = TYPE_STRING;
+  *Format.Type = 'C';
+  Format.Length = Long;
+  Format.Prec = 1;          // Case insensitive
+  Constant = true;          // TODO: check whether this is true indeed
+  Pname = NULL;
+  } // end of PRTBLK constructor
+
+/***********************************************************************/
+/*  ReadColumn: what this routine does is to return the partition ID.  */
+/***********************************************************************/
+void PRTBLK::ReadColumn(PGLOBAL g)
+  {
+  if (Pname == NULL) {
+    char   *p;
+
+    Pname = To_Tdb->GetDef()->GetStringCatInfo(g, "partname", "?");
+    p = strrchr(Pname, '#');
+    Value->SetValue_psz((p) ? p + 1 : Pname);
+    } // endif Pname
+
+  } // end of ReadColumn
+
 /***********************************************************************/
 /*  SIDBLK constructor for the SERVID special column.                  */
 /***********************************************************************/
@@ -365,7 +404,7 @@ SIDBLK::SIDBLK(PCOLUMN cp) : SPCBLK(cp)
 /***********************************************************************/
 /*  ReadColumn: what this routine does is to return the server ID.     */
 /***********************************************************************/
-void SIDBLK::ReadColumn(PGLOBAL g)
+void SIDBLK::ReadColumn(PGLOBAL)
   {
 //if (Sname == NULL) {
     Sname = (char*)To_Tdb->GetServer();
