@@ -157,23 +157,22 @@ bool CntCheckDB(PGLOBAL g, PHC handler, const char *pathname)
 /*  Returns valid: true if this is a table info.                       */
 /***********************************************************************/
 bool CntInfo(PGLOBAL g, PTDB tp, PXF info)
-  {
-  bool    b;
-  PTDBDOS tdbp= (PTDBDOS)tp;
+{
+  if (tp) {
+		bool    b = (tp->GetFtype() == RECFM_NAF);
+		PTDBDOS tdbp = b ? NULL : (PTDBDOS)tp;
 
-  if (tdbp) {
-    b= tdbp->GetFtype() != RECFM_NAF;
-    info->data_file_length= (b) ? (ulonglong)tdbp->GetFileLength(g) : 0;
+		info->data_file_length = (b) ? 0 : (ulonglong)tdbp->GetFileLength(g);
 
-    if (!b || info->data_file_length)
-      info->records= (unsigned)tdbp->Cardinality(g);
-//      info->records= (unsigned)tdbp->GetMaxSize(g);
+    if (b || info->data_file_length)
+      info->records= (unsigned)tp->Cardinality(g);
+//    info->records= (unsigned)tp->GetMaxSize(g);
     else
       info->records= 0;
 
 //  info->mean_rec_length= tdbp->GetLrecl();
     info->mean_rec_length= 0;
-    info->data_file_name= (b) ? tdbp->GetFile(g) : NULL;
+    info->data_file_name= (b) ? NULL : tdbp->GetFile(g);
     return true;
   } else {
     info->data_file_length= 0;
@@ -183,7 +182,7 @@ bool CntInfo(PGLOBAL g, PTDB tp, PXF info)
     return false;
   } // endif tdbp
 
-  } // end of CntInfo
+} // end of CntInfo
 
 /***********************************************************************/
 /*  GetTDB: Get the table description block of a CONNECT table.        */
@@ -332,9 +331,9 @@ bool CntOpenTable(PGLOBAL g, PTDB tdbp, MODE mode, char *c1, char *c2,
       } // endfor colp
 
     // Attach the updated columns list to the main table
-    ((PTDBASE)tdbp)->SetSetCols(utp->GetColumns());
+    tdbp->SetSetCols(utp->GetColumns());
   } else if (tdbp && mode == MODE_INSERT)
-    ((PTDBASE)tdbp)->SetSetCols(tdbp->GetColumns());
+    tdbp->SetSetCols(tdbp->GetColumns());
 
   // Now do open the physical table
   if (trace)
@@ -343,7 +342,7 @@ bool CntOpenTable(PGLOBAL g, PTDB tdbp, MODE mode, char *c1, char *c2,
 
 //tdbp->SetMode(mode);
 
-  if (del/* && ((PTDBASE)tdbp)->GetFtype() != RECFM_NAF*/) {
+  if (del/* && (tdbp->GetFtype() != RECFM_NAF*/) {
     // To avoid erasing the table when doing a partial delete
     // make a fake Next
 //    PDOSDEF ddp= new(g) DOSDEF;
@@ -436,7 +435,7 @@ RCODE CntReadNext(PGLOBAL g, PTDB tdbp)
 
   if (!tdbp)
     return RC_FX;
-  else if (((PTDBASE)tdbp)->GetKindex()) {
+  else if (tdbp->GetKindex()) {
     // Reading sequencially an indexed table. This happens after the
     // handler function records_in_range was called and MySQL decides
     // to quit using the index (!!!) Drop the index.
@@ -483,7 +482,7 @@ RCODE  CntWriteRow(PGLOBAL g, PTDB tdbp)
   {
   RCODE   rc;
   PCOL    colp;
-  PTDBASE tp= (PTDBASE)tdbp;
+//PTDBASE tp= (PTDBASE)tdbp;
 
   if (!tdbp)
     return RC_FX;
@@ -501,13 +500,13 @@ RCODE  CntWriteRow(PGLOBAL g, PTDB tdbp)
     } // endif rc
 
   // Store column values in table write buffer(s)
-  for (colp= tp->GetSetCols(); colp; colp= colp->GetNext())
+  for (colp= tdbp->GetSetCols(); colp; colp= colp->GetNext())
     if (!colp->GetColUse(U_VIRTUAL))
       colp->WriteColumn(g);
 
-  if (tp->IsIndexed())
+  if (tdbp->IsIndexed())
     // Index values must be sorted before updating
-    rc= (RCODE)((PTDBDOS)tp)->GetTxfp()->StoreValues(g, true);
+    rc= (RCODE)((PTDBDOS)tdbp)->GetTxfp()->StoreValues(g, true);
   else
     // Return result code from write operation
     rc= (RCODE)tdbp->WriteDB(g);
@@ -535,7 +534,7 @@ RCODE CntUpdateRow(PGLOBAL g, PTDB tdbp)
 RCODE  CntDeleteRow(PGLOBAL g, PTDB tdbp, bool all)
   {
   RCODE   rc;
-  PTDBASE tp= (PTDBASE)tdbp;
+//PTDBASE tp= (PTDBASE)tdbp;
 
   if (!tdbp || tdbp->GetMode() != MODE_DELETE)
     return RC_FX;
@@ -543,16 +542,16 @@ RCODE  CntDeleteRow(PGLOBAL g, PTDB tdbp, bool all)
     return RC_NF;
 
   if (all) {
-    if (((PTDBASE)tdbp)->GetDef()->Indexable())
+    if (tdbp->GetDef()->Indexable())
       ((PTDBDOS)tdbp)->Cardinal= 0;
 
     // Note: if all, this call will be done when closing the table
     rc= (RCODE)tdbp->DeleteDB(g, RC_FX);
-//} else if (tp->GetKindex() && !tp->GetKindex()->IsSorted() &&
-//           tp->Txfp->GetAmType() != TYPE_AM_DBF) {
-  } else if(tp->IsIndexed()) {
+//} else if (tdbp->GetKindex() && !((PTDBASE)tdbp)->GetKindex()->IsSorted() &&
+//           ((PTDBASE)tdbp)->Txfp->GetAmType() != TYPE_AM_DBF) {
+  } else if(tdbp->IsIndexed()) {
     // Index values must be sorted before updating
-    rc= (RCODE)((PTDBDOS)tp)->GetTxfp()->StoreValues(g, false);
+    rc= (RCODE)((PTDBDOS)tdbp)->GetTxfp()->StoreValues(g, false);
   } else // Return result code from delete operation
     rc= (RCODE)tdbp->DeleteDB(g, RC_OK);
 
@@ -565,7 +564,7 @@ RCODE  CntDeleteRow(PGLOBAL g, PTDB tdbp, bool all)
 int CntCloseTable(PGLOBAL g, PTDB tdbp, bool nox, bool abort)
   {
   int     rc= RC_OK;
-  TDBASE *tbxp= (PTDBASE)tdbp;
+//TDBASE *tbxp= (PTDBASE)tdbp;
 
   if (!tdbp)
     return rc;                           // Nothing to do
@@ -581,13 +580,13 @@ int CntCloseTable(PGLOBAL g, PTDB tdbp, bool nox, bool abort)
                            tdbp, tdbp->GetMode(), nox, abort);
 
   if (tdbp->GetMode() == MODE_DELETE && tdbp->GetUse() == USE_OPEN) {
-    if (tbxp->IsIndexed())
+    if (tdbp->IsIndexed())
       rc= ((PTDBDOS)tdbp)->GetTxfp()->DeleteSortedRows(g);
 
     if (!rc)
       rc= tdbp->DeleteDB(g, RC_EF);    // Specific A.M. delete routine
 
-  } else if (tbxp->GetMode() == MODE_UPDATE && tbxp->IsIndexed())
+  } else if (tdbp->GetMode() == MODE_UPDATE && tdbp->IsIndexed())
     rc= ((PTDBDOX)tdbp)->Txfp->UpdateSortedRows(g);
 
   switch(rc) {
@@ -595,7 +594,7 @@ int CntCloseTable(PGLOBAL g, PTDB tdbp, bool nox, bool abort)
       abort= true;
       break;
     case RC_INFO:
-      PushWarning(g, tbxp);
+      PushWarning(g, tdbp);
       break;
     } // endswitch rc
 
@@ -631,11 +630,13 @@ int CntCloseTable(PGLOBAL g, PTDB tdbp, bool nox, bool abort)
   if (trace > 1)
     printf("About to reset opt\n");
 
-  // Make all the eventual indexes
-  tbxp= (TDBDOX*)tdbp;
-  tbxp->ResetKindex(g, NULL);
-  tbxp->SetKey_Col(NULL);
-  rc= tbxp->ResetTableOpt(g, true, tbxp->GetDef()->Indexable() == 1);
+	if (!tdbp->IsRemote()) {
+		// Make all the eventual indexes
+		PTDBDOX tbxp = (PTDBDOX)tdbp;
+		tbxp->ResetKindex(g, NULL);
+		tbxp->SetKey_Col(NULL);
+		rc = tbxp->ResetTableOpt(g, true, tbxp->GetDef()->Indexable() == 1);
+	  } // endif remote
 
  err:
   if (trace > 1)
@@ -657,10 +658,10 @@ int CntIndexInit(PGLOBAL g, PTDB ptdb, int id, bool sorted)
 
   if (!ptdb)
     return -1;
-  else if (!((PTDBASE)ptdb)->GetDef()->Indexable()) {
+  else if (!ptdb->GetDef()->Indexable()) {
     sprintf(g->Message, MSG(TABLE_NO_INDEX), ptdb->GetName());
     return 0;
-  } else if (((PTDBASE)ptdb)->GetDef()->Indexable() == 3) {
+  } else if (ptdb->GetDef()->Indexable() == 3) {
     return 1;
   } else
     tdbp= (PTDBDOX)ptdb;
@@ -745,14 +746,14 @@ RCODE CntIndexRead(PGLOBAL g, PTDB ptdb, OPVAL op,
   if (!ptdb)
     return RC_FX;
   else
-    x= ((PTDBASE)ptdb)->GetDef()->Indexable();
+    x= ptdb->GetDef()->Indexable();
 
   if (!x) {
     sprintf(g->Message, MSG(TABLE_NO_INDEX), ptdb->GetName());
     return RC_FX;
   } else if (x == 2) {
     // Remote index
-    if (ptdb->ReadKey(g, op, kr))
+    if (op != OP_SAME && ptdb->ReadKey(g, op, kr))
       return RC_FX;
 
     goto rnd;
@@ -875,7 +876,7 @@ int CntIndexRange(PGLOBAL g, PTDB ptdb, const uchar* *key, uint *len,
   if (!ptdb)
     return -1;
 
-  x= ((PTDBASE)ptdb)->GetDef()->Indexable();
+  x= ptdb->GetDef()->Indexable();
 
   if (!x) {
     sprintf(g->Message, MSG(TABLE_NO_INDEX), ptdb->GetName());
