@@ -1,7 +1,7 @@
 /*************** Tabxml H Declares Source Code File (.H) ***************/
-/*  Name: TABXML.H    Version 1.6                                      */
+/*  Name: TABXML.H    Version 1.7                                      */
 /*                                                                     */
-/*  (C) Copyright to the author Olivier BERTRAND          2007-2013    */
+/*  (C) Copyright to the author Olivier BERTRAND          2007-2016    */
 /*                                                                     */
 /*  This file contains the XML table classes declares.                 */
 /***********************************************************************/
@@ -16,6 +16,8 @@ typedef class XMLCOL *PXMLCOL;
 /***********************************************************************/
 class DllExport XMLDEF : public TABDEF {  /* Logical table description */
   friend class TDBXML;
+  friend class TDBXCT;
+  friend PQRYRES XMLColumns(PGLOBAL, char*, char*, PTOS, bool);
  public:
   // Constructor
    XMLDEF(void);
@@ -40,12 +42,15 @@ class DllExport XMLDEF : public TABDEF {  /* Logical table description */
   char   *DefNs;                  /* Dummy name of default namespace   */
   char   *Attrib;                 /* Table node attributes             */
   char   *Hdattr;                 /* Header node attributes            */
-  int     Coltype;                /* Default column type               */
+	char   *Entry;						      /* Zip entry name or pattern				 */
+	int     Coltype;                /* Default column type               */
   int     Limit;                  /* Limit of multiple values          */
   int     Header;                 /* n first rows are header rows      */
   bool    Xpand;                  /* Put multiple tags in several rows */
   bool    Usedom;                 /* True: DOM, False: libxml2         */
-  }; // end of XMLDEF
+	bool    Zipped;                 /* True: Zipped XML file(s)          */
+	bool    Mulentries;             /* True: multiple entries in zip file*/
+}; // end of XMLDEF
 
 #if defined(INCLUDE_TDBXML)
 /***********************************************************************/
@@ -55,6 +60,7 @@ class DllExport TDBXML : public TDBASE {
   friend class XMLCOL;
   friend class XMULCOL;
   friend class XPOSCOL;
+  friend PQRYRES XMLColumns(PGLOBAL, char*, char*, PTOS, bool);
  public:
   // Constructor
   TDBXML(PXMLDEF tdp);
@@ -65,7 +71,7 @@ class DllExport TDBXML : public TDBASE {
   virtual PTDB  Duplicate(PGLOBAL g) {return (PTDB)new(g) TDBXML(this);}
 
   // Methods
-  virtual PTDB  CopyOne(PTABS t);
+  virtual PTDB  Clone(PTABS t);
   virtual int   GetRecpos(void);
   virtual int   GetProgCur(void) {return N;}
   virtual PSZ   GetFile(PGLOBAL g) {return Xfile;}
@@ -81,7 +87,7 @@ class DllExport TDBXML : public TDBASE {
 
   // Database routines
   virtual PCOL  MakeCol(PGLOBAL g, PCOLDEF cdp, PCOL cprec, int n);
-  virtual PCOL  InsertSpecialColumn(PGLOBAL g, PCOL colp);
+  virtual PCOL  InsertSpecialColumn(PCOL colp);
 //virtual int   GetMaxSame(PGLOBAL g) {return (Xpand) ? Limit : 1;}
   virtual int   Cardinality(PGLOBAL g);
   virtual int   GetMaxSize(PGLOBAL g);
@@ -119,7 +125,9 @@ class DllExport TDBXML : public TDBASE {
   bool    Bufdone;                  // True when column buffers allocated
   bool    Nodedone;                 // True when column nodes allocated
   bool    Void;                     // True if the file does not exist
-  char   *Xfile;                    // The XML file
+	bool    Zipped;                   // True if Zipped XML file(s)
+	bool    Mulentries;               // True if multiple entries in zip file
+	char   *Xfile;                    // The XML file
   char   *Enc;                      // New XML table file encoding
   char   *Tabname;                  // Name of Table node
   char   *Rowname;                  // Name of first level nodes
@@ -130,7 +138,8 @@ class DllExport TDBXML : public TDBASE {
   char   *DefNs;                    // Dummy name of default namespace
   char   *Attrib;                   // Table node attribut(s)
   char   *Hdattr;                   // Header node attribut(s)
-  int     Coltype;                  // Default column type
+	char   *Entry;						        // Zip entry name or pattern
+	int     Coltype;                  // Default column type
   int     Limit;                    // Limit of multiple values
   int     Header;                   // n first rows are header rows
   int     Multiple;                 // If multiple files
@@ -237,4 +246,23 @@ class XPOSCOL : public XMLCOLX {
   virtual void ReadColumn(PGLOBAL g);
   virtual void WriteColumn(PGLOBAL g);
   }; // end of class XPOSCOL
+
+/***********************************************************************/
+/*  This is the class declaration for the XML catalog table.           */
+/***********************************************************************/
+class TDBXCT : public TDBCAT {
+ public:
+  // Constructor
+  TDBXCT(PXMLDEF tdp);
+
+ protected:
+  // Specific routines
+  virtual PQRYRES GetResult(PGLOBAL g);
+
+  // Members
+  PTOS  Topt;
+  char *Db;
+  char *Tabn;
+  }; // end of class TDBXCT
+
 #endif // INCLUDE_TDBXML
