@@ -245,7 +245,7 @@ ib_open_table_by_id(
 		dict_mutex_enter_for_mysql();
 	}
 
-	table = dict_table_open_on_id(table_id, FALSE, DICT_TABLE_OP_NORMAL);
+	table = dict_table_open_on_id(table_id, TRUE, DICT_TABLE_OP_NORMAL);
 
 	if (table != NULL && table->ibd_file_missing) {
 		table = NULL;
@@ -2116,6 +2116,10 @@ ib_cursor_moveto(
 
 	n_fields = dict_index_get_n_ordering_defined_by_user(prebuilt->index);
 
+	if (n_fields > dtuple_get_n_fields(tuple->ptr)) {
+		n_fields = dtuple_get_n_fields(tuple->ptr);
+	}
+
 	dtuple_set_n_fields(search_tuple, n_fields);
 	dtuple_set_n_fields_cmp(search_tuple, n_fields);
 
@@ -3753,14 +3757,14 @@ ib_table_truncate(
 	if (trunc_err == DB_SUCCESS) {
 		ut_a(ib_trx_state(ib_trx) == static_cast<ib_trx_state_t>(
 			TRX_STATE_NOT_STARTED));
-
-		err = ib_trx_release(ib_trx);
-		ut_a(err == DB_SUCCESS);
 	} else {
 		err = ib_trx_rollback(ib_trx);
 		ut_a(err == DB_SUCCESS);
 	}
 
+	err = ib_trx_release(ib_trx);
+	ut_a(err == DB_SUCCESS);
+
 	/* Set the memcached_sync_count back. */
 	if (table != NULL && memcached_sync != 0) {
 		dict_mutex_enter_for_mysql();
