@@ -1,6 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 2000, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2017, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -41,9 +42,6 @@ struct SysIndexCallback;
 
 extern ibool row_rollback_on_timeout;
 
-extern uint	srv_compressed_columns_zip_level;
-extern ulong	srv_compressed_columns_threshold;
-
 struct row_prebuilt_t;
 
 /*******************************************************************//**
@@ -55,48 +53,6 @@ row_mysql_prebuilt_free_blob_heap(
 	row_prebuilt_t*	prebuilt);	/*!< in: prebuilt struct of a
 					ha_innobase:: table handle */
 
-/** Frees the compress heap in prebuilt when no longer needed. */
-UNIV_INTERN
-void
-row_mysql_prebuilt_free_compress_heap(
-	row_prebuilt_t*	prebuilt);	/*!< in: prebuilt struct of a
-					ha_innobase:: table handle */
-
-/** Uncompress blob/text/varchar column using zlib
-@return pointer to the uncompressed data */
-const byte*
-row_decompress_column(
-	const byte*	data,	/*!< in: data in innodb(compressed) format */
-	ulint		*len,	/*!< in: data length; out: length of
-				decompressed data*/
-	const byte*	dict_data,
-				/*!< in: optional dictionary data used for
-				decompression */
-	ulint		dict_data_len,
-				/*!< in: optional dictionary data length */
-	row_prebuilt_t*	prebuilt);
-				/*!< in: use prebuilt->compress_heap only
-				here*/
-
-/** Compress blob/text/varchar column using zlib
-@return pointer to the compressed data */
-byte*
-row_compress_column(
-	const byte*	data,	/*!< in: data in mysql(uncompressed)
-				format */
-	ulint		*len,	/*!< in: data length; out: length of
-				compressed data*/
-	ulint		lenlen,	/*!< in: bytes used to store the length of
-				data */
-	const byte*	dict_data,
-				/*!< in: optional dictionary data used for
-				compression */
-	ulint		dict_data_len,
-				/*!< in: optional dictionary data length */
-	row_prebuilt_t*	prebuilt);
-				/*!< in: use prebuilt->compress_heap only
-				here*/
-
 /*******************************************************************//**
 Stores a >= 5.0.3 format true VARCHAR length to dest, in the MySQL row
 format.
@@ -135,21 +91,10 @@ row_mysql_store_blob_ref(
 				to 4 bytes */
 	const void*	data,	/*!< in: BLOB data; if the value to store
 				is SQL NULL this should be NULL pointer */
-	ulint		len,	/*!< in: BLOB length; if the value to store
+	ulint		len);	/*!< in: BLOB length; if the value to store
 				is SQL NULL this should be 0; remember
 				also to set the NULL bit in the MySQL record
 				header! */
-	bool		need_decompression,
-				/*!< in: if the data need to be compressed*/
-	const byte*	dict_data,
-				/*!< in: optional compression dictionary
-				data */
-	ulint		dict_data_len,
-				/*!< in: optional compression dictionary data
-				length */
-	row_prebuilt_t*	prebuilt);
-				/*<! in: use prebuilt->compress_heap only
-				here */
 /*******************************************************************//**
 Reads a reference to a BLOB in the MySQL format.
 @return	pointer to BLOB data */
@@ -160,17 +105,8 @@ row_mysql_read_blob_ref(
 	ulint*		len,		/*!< out: BLOB length */
 	const byte*	ref,		/*!< in: BLOB reference in the
 					MySQL format */
-	ulint		col_len,	/*!< in: BLOB reference length
+	ulint		col_len);	/*!< in: BLOB reference length
 					(not BLOB length) */
-	bool		need_compression,
-					/*!< in: if the data need to be
-					compressed*/
-	const byte*	dict_data,	/*!< in: optional compression
-					dictionary data */
-	ulint		dict_data_len,	/*!< in: optional compression
-					dictionary data length */
-	row_prebuilt_t*	prebuilt);	/*!< in: use prebuilt->compress_heap
-					only here */
 /**************************************************************//**
 Pad a column with spaces. */
 UNIV_INTERN
@@ -218,16 +154,7 @@ row_mysql_store_col_in_innobase_format(
 					necessarily the length of the actual
 					payload data; if the column is a true
 					VARCHAR then this is irrelevant */
-	ulint		comp,		/*!< in: nonzero=compact format */
-	bool		need_compression,
-					/*!< in: if the data need to be
-					compressed */
-	const byte*	dict_data,	/*!< in: optional compression
-					dictionary data */
-	ulint		dict_data_len,	/*!< in: optional compression
-					dictionary data length */
-	row_prebuilt_t*	prebuilt);	/*!< in: use prebuilt->compress_heap
-					only here */
+	ulint		comp);		/*!< in: nonzero=compact format */
 /****************************************************************//**
 Handles user errors and lock waits detected by the database engine.
 @return true if it was a lock wait and we should continue running the
@@ -721,8 +648,6 @@ struct mysql_row_templ_t {
 	ulint	is_unsigned;		/*!< if a column type is an integer
 					type and this field is != 0, then
 					it is an unsigned integer type */
-	bool		compressed;	/*!< if column format is compressed */
-	LEX_CSTRING	zip_dict_data;	/*!< associated compression dictionary */
 };
 
 #define MYSQL_FETCH_CACHE_SIZE		8
@@ -920,8 +845,6 @@ struct row_prebuilt_t {
 					in fetch_cache */
 	mem_heap_t*	blob_heap;	/*!< in SELECTS BLOB fields are copied
 					to this heap */
-	mem_heap_t*	compress_heap;  /*!< memory heap used to compress
-					/decompress blob column*/
 	mem_heap_t*	old_vers_heap;	/*!< memory heap where a previous
 					version is built in consistent read */
 	bool		in_fts_query;	/*!< Whether we are in a FTS query */
